/**
 *
 */
package eu.etaxonomy.taxeditor.editor.view.descriptive;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.cdm.model.description.IDescribable;
import eu.etaxonomy.taxeditor.editor.UsageTermCollection;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainerTree;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.TermStore;

/**
 * <p>DescriptiveContentProvider class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @version $Id: $
 */
public class DescriptiveContentProvider implements ITreeContentProvider {

	protected static final Object[] NO_CHILDREN = new Object[0];
	protected Map<DescriptionBase<?>, FeatureNodeContainerTree> featureNodeContainerCache;

	/**
	 * <p>Constructor for DescriptiveContentProvider.</p>
	 *
	 * @param featureNodeContainerCache a {@link java.util.Map} object.
	 */
	public DescriptiveContentProvider(Map<DescriptionBase<?>, FeatureNodeContainerTree> featureNodeContainerCache) {
		this.featureNodeContainerCache = featureNodeContainerCache;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
    public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof IDescribable<?>) {
			return getDescriptions((IDescribable<?>) parentElement).toArray();
		}
		else if (parentElement instanceof DescriptionBase<?>) {
			if ( ! ((DescriptionBase<?>) parentElement).isImageGallery()) {
			    DescriptionBase<?> description = (DescriptionBase<?>) parentElement;

				FeatureNodeContainerTree containerTree = getContainerTreeForDesription(description);

				return containerTree.getRoot().getChildren().toArray();
			}
		}
		else if (parentElement instanceof FeatureNodeContainer){
			FeatureNodeContainer container = (FeatureNodeContainer) parentElement;
			if(container.isLeaf()){
				return container.getDescriptionElements().toArray();
			}else{
				return container.getChildren().toArray();
			}
		}

		return NO_CHILDREN;
	}

	private FeatureNodeContainerTree getContainerTreeForDesription(DescriptionBase<?> description){
		if(! featureNodeContainerCache.containsKey(description)){
			FeatureNodeContainerTree containerTree = new FeatureNodeContainerTree(description, getFeatureTree(description));
			featureNodeContainerCache.put(description, containerTree);
		}
		return featureNodeContainerCache.get(description);
	}

	/** {@inheritDoc} */
	@Override
	public boolean hasChildren(Object element) {
		if (element instanceof DescriptionBase<?>){
		    DescriptionBase<?> description = (DescriptionBase<?>) element;
			FeatureNodeContainerTree containerTree = featureNodeContainerCache.get(description);
			if(containerTree != null && containerTree.getRoot() != null){
				return containerTree.getRoot().getChildren().size() != 0;
			}
		}
		return getChildren(element).length != 0;
	}

	/**
	 * Retrieves the feature tree associated with the given description
	 *
	 * TODO as of now this is always the same thing because feature trees may not be associated
	 * to descriptions yet.
	 *
	 * @param description
	 * @return
	 */
	private FeatureTree getFeatureTree(DescriptionBase description){
		FeatureTree featureTree = null;

		// TODO change this to the feature tree associated with this taxon description
		if (description.hasStructuredData()){
			featureTree = PreferencesUtil.getDefaultFeatureTreeForStructuredDescription();
		}else{
			featureTree = PreferencesUtil.getDefaultFeatureTreeForTextualDescription();
		}

		// create a transient tree with all features if none was selected
		if(featureTree == null){
			featureTree = FeatureTree.NewInstance(TermStore.getTerms(Feature.class));
		}

		return featureTree;
	}

    /**
     * Get all descriptions associated with the given object
     * @param parentElement
     * @return
     */
    protected List<DescriptionBase<?>> getDescriptions(IDescribable<?> parentElement) {
        Set<? extends DescriptionBase<?>> elementDescriptions = parentElement.getDescriptions();
        List<DescriptionBase<?>> resultDescriptions = new ArrayList<DescriptionBase<?>>();
        for(DescriptionBase<?> description : elementDescriptions){
			if(! description.isImageGallery()){
				MarkerType useMarkertype = (MarkerType) CdmStore.getService(ITermService.class).find(UsageTermCollection.uuidUseMarkerType);
				Set<Marker> descriptionMarkers = description.getMarkers();
				if(descriptionMarkers != null && !descriptionMarkers.isEmpty()) {
					for (Marker marker: descriptionMarkers) {
						if(!(marker.getMarkerType().equals(useMarkertype))) {
							resultDescriptions.add(description);
						}
					}
				}
				else {
					resultDescriptions.add(description);
				}
			}
		}
        return resultDescriptions;
    }

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
    public Object getParent(Object element) {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
    public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	/**
	 * <p>dispose</p>
	 */
	@Override
    public void dispose() {
		featureNodeContainerCache.clear();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

}
